
#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <string.h>
#include <sys/ioctl.h>
#include <linux/spi/spidev.h>
#include <errno.h>

#include "SPI.h"

#include "RFIDReader_readWriteReg.h"


/** Credit:
 * Links working as of Apr 2024
 * (1) CMPT 433 "SPI Guide for Linux C on BeagleBone Green" (Cruikshank, Garofano, Joshi, Vanloo)
 *      * Provided starter code for this file, SPI, read/write registers, and links to other references
 * (2) MFRC522 Datasheet (https://www.nxp.com/docs/en/data-sheet/MFRC522.pdf)
 * (3) ISO/IEC 14443 Type A info (https://nfc-tools.github.io/resources/standards/iso14443A/)
 * (4) linux-spi-rfid-rc522 library (https://github.com/cazure/linux-spi-rfid-rc522)
 *      * Provided some header variables (error codes)
 * (5) RC522 Arduino tutorial (https://www.youtube.com/watch?v=gTQmLkwnUxk)
*/


/* Function prototypes */
void RFIDReader_init();
static void rfid_fifo_write(uint8_t *buff, uint8_t size);
static void rfid_fifo_read(uint8_t *buff, uint8_t size);


/* This is included in the SPI guide */
void RFIDReader_writeReg(enum MFRC522_Register regAddr, uint8_t regData)
{
    const int NUM_BYTES = 2;

    uint8_t sendBuf[NUM_BYTES];
    // Address byte (See section 8.1.2.3 of Doc)
    sendBuf[0] = (REG_WRITE_OP_MASK | (regAddr << REG_ADDR_BIT));
    // Data byte
    sendBuf[1] = regData;

	uint8_t recvBuf[NUM_BYTES] = {0};

    SPI_transfer(spiFileDesc, sendBuf, recvBuf, NUM_BYTES);
}


/* This is included in the SPI guide */
uint8_t RFIDReader_readReg(enum MFRC522_Register regAddr)
{
    const int NUM_BYTES = 2;

    uint8_t sendBuf[NUM_BYTES];
    // Address byte (See section 8.1.2.3 of Doc)
    sendBuf[0] = (REG_READ_OP_MASK | (regAddr << REG_ADDR_BIT));
    // Second address byte (set 0 to stop reading) 
    sendBuf[1] = 0x00;

	uint8_t recvBuf[NUM_BYTES] = {0};

    SPI_transfer(spiFileDesc, sendBuf, recvBuf, NUM_BYTES);

    // First received byte will be blank
    return recvBuf[1];
}


/* Initialization function */
void RFIDReader_init() {

    // Assuming SPI and GPIO are initialized
    
    // See datasheet §8.5 for prescaler calculations
    // Partial credit (for general process): github.com/miguelbalboa/rfid/blob/master/src/MFRC522.cpp
    RFIDReader_writeReg(TModeReg, (1 << TMODEREG_TAUTO_BIT) | (0x04 << TMODEREG_TPRESCALER_HI_BIT));
    RFIDReader_writeReg(TPrescalerReg, 0x00);
    RFIDReader_writeReg(TReloadRegH, 0x01);
    RFIDReader_writeReg(TReloadRegL, 0x49);

    // Mode settings (§9.3.2.2)
    RFIDReader_writeReg(ModeReg, (1 << MODEREG_TXWAITRF_BIT) | (1 << MODEREG_POLMFIN_BIT)  | (1 << MODEREG_CRCPRESET_BIT));

    // ASK settings (§9.3.2.6)
    RFIDReader_writeReg(TxASKReg, 1 << TXASKREG_FORCE100ASK_BIT);

    // Antenna on
    uint8_t TxControl = RFIDReader_readReg(TxControlReg);
    RFIDReader_writeReg(TxControlReg, (TxControl | TXCONTROLREG_ANTENNA_ON_MASK));
}


/* Write to the FIFO buffer (see §8.3) */
static void rfid_fifo_write(uint8_t *buff, uint8_t size) {

    uint8_t payload = size + 1;
    uint8_t sendBuf[payload];
    uint8_t recvBuf[payload];

    // Setup the first byte of the send buffer as the command/address byte
    sendBuf[0] = (REG_WRITE_OP_MASK | (FIFODataReg << REG_ADDR_BIT));

    // Copy buff into sendBuf, starting at sendBuf[1]
    memcpy(sendBuf + 1, buff, size);

    // Perform the SPI transfer
    SPI_transfer(spiFileDesc, sendBuf, recvBuf, payload);

    // Assuming recvBuf unused
}


/* Read from the FIFO buffer (see §8.3) */
static void rfid_fifo_read(uint8_t *buff, uint8_t size) {

    uint8_t payload = size + 1;
    uint8_t sendBuf[payload];
    uint8_t recvBuf[payload];

    // Send buffer for read operation
    memset(sendBuf, (REG_READ_OP_MASK | (FIFODataReg << REG_ADDR_BIT)), payload - 1);
    sendBuf[payload - 1] = 0; // Set last byte to stop reading

    SPI_transfer(spiFileDesc, sendBuf, recvBuf, payload);

    // Copy received data to output buffer
    memcpy(buff, recvBuf + 1, size);
}
